@@ -9,11 +9,14 @@ Module implementing a dialog showing the code formatting progress and the result
 
 import copy
 import datetime
+import multiprocessing
 import pathlib
 
+from dataclasses import dataclass
+
 import black
 
-from PyQt6.QtCore import pyqtSlot, Qt, QCoreApplication
+from PyQt6.QtCore import pyqtSignal, pyqtSlot, Qt, QCoreApplication, QObject
 from PyQt6.QtWidgets import (
     QAbstractButton,
     QDialog,
@@ -31,6 +34,7 @@ from .BlackDiffWidget import BlackDiffWidget
 from .BlackFormattingAction import BlackFormattingAction
 
 import Utilities
+import Preferences
 
 
 class BlackFormattingDialog(QDialog, Ui_BlackFormattingDialog):
@@ -76,11 +80,10 @@ class BlackFormattingDialog(QDialog, Ui_BlackFormattingDialog):
 
         self.statisticsGroup.setVisible(False)
 
-        self.__report = BlackReport(self)
-        self.__report.check = action is BlackFormattingAction.Check
-        self.__report.diff = action is BlackFormattingAction.Diff
+        self.__statistics = BlackStatistics()
 
         self.__config = copy.deepcopy(configuration)
+        self.__config["__action__"] = action  # needed by the workers
         self.__project = project
         self.__action = action
 
@@ -96,8 +99,11 @@ class BlackFormattingDialog(QDialog, Ui_BlackFormattingDialog):
         self.show()
         QCoreApplication.processEvents()
 
-        self.__files = self.__filterFiles(filesList)
-        self.__formatFiles()
+        files = self.__filterFiles(filesList)
+        if len(files) > 1:
+            self.__formatManyFiles(files)
+        elif len(files) == 1:
+            self.__formatOneFile(files[0])
 
     def __filterFiles(self, filesList):
         """
@@ -123,7 +129,7 @@ class BlackFormattingDialog(QDialog, Ui_BlackFormattingDialog):
             for filterRegExp in filterRegExps:
                 filterMatch = filterRegExp.search(file)
                 if filterMatch and filterMatch.group(0):
-                    self.__report.path_ignored(file)
+                    self.__handleBlackFormattingResult("ignored", file, "")
                     break
             else:
                 files.append(file)
@@ -185,20 +191,23 @@ class BlackFormattingDialog(QDialog, Ui_BlackFormattingDialog):
         make them visible.
         """
         self.reformattedLabel.setText(
-            self.tr("reformatted")
+            self.tr("Reformatted:")
             if self.__action is BlackFormattingAction.Format
-            else self.tr("would reformat")
+            else self.tr("Would Reformat:")
         )
 
         total = self.progressBar.maximum()
-        processed = total - self.__report.ignored_count
 
         self.totalCountLabel.setText("{0:n}".format(total))
-        self.excludedCountLabel.setText("{0:n}".format(self.__report.ignored_count))
-        self.failuresCountLabel.setText("{0:n}".format(self.__report.failure_count))
-        self.processedCountLabel.setText("{0:n}".format(processed))
-        self.reformattedCountLabel.setText("{0:n}".format(self.__report.change_count))
-        self.unchangedCountLabel.setText("{0:n}".format(self.__report.same_count))
+        self.excludedCountLabel.setText("{0:n}".format(self.__statistics.ignoreCount))
+        self.failuresCountLabel.setText("{0:n}".format(self.__statistics.failureCount))
+        self.processedCountLabel.setText(
+            "{0:n}".format(self.__statistics.processedCount)
+        )
+        self.reformattedCountLabel.setText(
+            "{0:n}".format(self.__statistics.changeCount)
+        )
+        self.unchangedCountLabel.setText("{0:n}".format(self.__statistics.sameCount))
 
         self.statisticsGroup.setVisible(True)
 
@@ -239,41 +248,147 @@ class BlackFormattingDialog(QDialog, Ui_BlackFormattingDialog):
                 self.__diffDialog = BlackDiffWidget()
             self.__diffDialog.showDiff(item.data(0, BlackFormattingDialog.DataRole))
 
-    def addResultEntry(self, status, fileName, isError=False, data=None):
+    def __formatManyFiles(self, files):
         """
-        Public method to add an entry to the result list.
+        Private method to format the list of files according the configuration using
+        multiple processes in parallel.
 
-        @param status status of the operation
-        @type str
-        @param fileName name of the processed file
-        @type str
-        @param isError flag indicating that data contains an error message (defaults to
-            False)
-        @type bool (optional)
-        @param data associated data (diff or error message) (defaults to None)
-        @type str (optional)
+        @param files list of files to be processed
+        @type list of str
         """
-        if self.__project:
-            fileName = self.__project.getRelativePath(fileName)
+        maxProcesses = Preferences.getUI("BackgroundServiceProcesses")
+        if maxProcesses == 0:
+            # determine based on CPU count
+            try:
+                NumberOfProcesses = multiprocessing.cpu_count()
+                if NumberOfProcesses >= 1:
+                    NumberOfProcesses -= 1
+            except NotImplementedError:
+                NumberOfProcesses = 1
+        else:
+            NumberOfProcesses = maxProcesses
 
-        itm = QTreeWidgetItem(self.resultsList, [status, fileName])
-        if data:
-            itm.setData(
-                0, BlackFormattingDialog.DataTypeRole, "error" if isError else "diff"
+        # Create queues
+        taskQueue = multiprocessing.Queue()
+        doneQueue = multiprocessing.Queue()
+
+        # Submit tasks (initially two times the number of processes)
+        tasks = len(files)
+        initialTasks = min(2 * NumberOfProcesses, tasks)
+        for _ in range(initialTasks):
+            file = files.pop(0)
+            relSrc = self.__project.getRelativePath(str(file)) if self.__project else ""
+            taskQueue.put((file, relSrc))
+
+        # Start worker processes
+        workers = [
+            multiprocessing.Process(
+                target=self.formattingWorkerTask,
+                args=(taskQueue, doneQueue, self.__config),
             )
-            itm.setData(0, BlackFormattingDialog.DataRole, data)
+            for _ in range(NumberOfProcesses)
+        ]
+        for worker in workers:
+            worker.start()
 
-        self.progressBar.setValue(self.progressBar.value() + 1)
+        # Get the results from the worker tasks
+        for _ in range(tasks):
+            result = doneQueue.get()
+            self.__handleBlackFormattingResult(
+                result.status, result.filename, result.data
+            )
 
-        QCoreApplication.processEvents()
+            if self.__cancelled:
+                break
 
-    def __formatFiles(self):
+            if files:
+                file = files.pop(0)
+                relSrc = (
+                    self.__project.getRelativePath(str(file)) if self.__project else ""
+                )
+                taskQueue.put((file, relSrc))
+
+        # Tell child processes to stop
+        for _ in range(NumberOfProcesses):
+            taskQueue.put("STOP")
+
+        for worker in workers:
+            worker.join()
+            worker.close()
+
+        taskQueue.close()
+        doneQueue.close()
+
+        self.__finish()
+
+    @staticmethod
+    def formattingWorkerTask(inputQueue, outputQueue, config):
+        """
+        Static method acting as the parallel worker for the formatting task.
+
+        @param inputQueue input queue
+        @type multiprocessing.Queue
+        @param outputQueue output queue
+        @type multiprocessing.Queue
+        @param config dictionary containing the configuration parameters
+        @type dict
+        """
+        report = BlackMultiprocessingReport(outputQueue)
+        report.check = config["__action__"] is BlackFormattingAction.Check
+        report.diff = config["__action__"] is BlackFormattingAction.Diff
+
+        versions = (
+            {black.TargetVersion[target.upper()] for target in config["target-version"]}
+            if config["target-version"]
+            else set()
+        )
+
+        mode = black.Mode(
+            target_versions=versions,
+            line_length=int(config["line-length"]),
+            string_normalization=not config["skip-string-normalization"],
+            magic_trailing_comma=not config["skip-magic-trailing-comma"],
+        )
+
+        if config["__action__"] is BlackFormattingAction.Diff:
+            for file, relSrc in iter(inputQueue.get, "STOP"):
+                BlackFormattingDialog.__diffFormatFile(
+                    pathlib.Path(file),
+                    fast=False,
+                    mode=mode,
+                    report=report,
+                    relSrc=relSrc,
+                )
+        else:
+            writeBack = black.WriteBack.from_configuration(
+                check=config["__action__"] is BlackFormattingAction.Check,
+                diff=config["__action__"] is BlackFormattingAction.Diff,
+            )
+
+            for file, _relSrc in iter(inputQueue.get, "STOP"):
+                black.reformat_one(
+                    pathlib.Path(file),
+                    fast=False,
+                    write_back=writeBack,
+                    mode=mode,
+                    report=report,
+                )
+
+    def __formatOneFile(self, file):
         """
         Private method to format the list of files according the configuration.
+
+        @param file name of the file to be processed
+        @type str
         """
+        report = BlackReport(self)
+        report.check = self.__config["__action__"] is BlackFormattingAction.Check
+        report.diff = self.__config["__action__"] is BlackFormattingAction.Diff
+        report.result.connect(self.__handleBlackFormattingResult)
+
         writeBack = black.WriteBack.from_configuration(
-            check=self.__action is BlackFormattingAction.Check,
-            diff=self.__action is BlackFormattingAction.Diff,
+            check=self.__config["__action__"] is BlackFormattingAction.Check,
+            diff=self.__config["__action__"] is BlackFormattingAction.Diff,
         )
 
         versions = (
@@ -292,28 +407,26 @@ class BlackFormattingDialog(QDialog, Ui_BlackFormattingDialog):
             magic_trailing_comma=not self.__config["skip-magic-trailing-comma"],
         )
 
-        for file in self.__files:
-            if self.__action is BlackFormattingAction.Diff:
-                self.__diffFormatFile(
-                    pathlib.Path(file), fast=False, mode=mode, report=self.__report
-                )
-            else:
-                black.reformat_one(
-                    pathlib.Path(file),
-                    fast=False,
-                    write_back=writeBack,
-                    mode=mode,
-                    report=self.__report,
-                )
-
-            if self.__cancelled:
-                break
+        if self.__action is BlackFormattingAction.Diff:
+            relSrc = self.__project.getRelativePath(str(file)) if self.__project else ""
+            self.__diffFormatFile(
+                pathlib.Path(file), fast=False, mode=mode, report=report, relSrc=relSrc
+            )
+        else:
+            black.reformat_one(
+                pathlib.Path(file),
+                fast=False,
+                write_back=writeBack,
+                mode=mode,
+                report=report,
+            )
 
         self.__finish()
 
-    def __diffFormatFile(self, src, fast, mode, report):
+    @staticmethod
+    def __diffFormatFile(src, fast, mode, report, relSrc=""):
         """
-        Private method to check, if the given files need to be reformatted, and generate
+        Static method to check, if the given files need to be reformatted, and generate
         a unified diff.
 
         @param src path of file to be checked
@@ -324,6 +437,8 @@ class BlackFormattingDialog(QDialog, Ui_BlackFormattingDialog):
         @type black.Mode
         @param report reference to the report object
         @type BlackReport
+        @param relSrc name of the file relative to the project (defaults to "")
+        @type str (optional)
         """
         then = datetime.datetime.utcfromtimestamp(src.stat().st_mtime)
         with open(src, "rb") as buf:
@@ -334,9 +449,7 @@ class BlackFormattingDialog(QDialog, Ui_BlackFormattingDialog):
             report.done(src, black.Changed.NO)
             return
 
-        fileName = str(src)
-        if self.__project:
-            fileName = self.__project.getRelativePath(fileName)
+        fileName = relSrc if bool(relSrc) else str(src)
 
         now = datetime.datetime.utcnow()
         srcName = f"{fileName}\t{then} +0000"
@@ -370,24 +483,103 @@ class BlackFormattingDialog(QDialog, Ui_BlackFormattingDialog):
                 and itm.text(BlackFormattingDialog.StatusColumn) != status
             )
 
+    @pyqtSlot(str, str, str)
+    def __handleBlackFormattingResult(self, status, filename, data):
+        """
+        Private slot to handle the result of a black reformatting action.
 
-class BlackReport(black.Report):
+        @param status status of the performed action (one of 'changed', 'unchanged',
+            'unmodified', 'failed' or 'ignored')
+        @type str
+        @param filename name of the processed file
+        @type str
+        @param data action data (error message or unified diff)
+        @type str
+        """
+        isError = False
+
+        if status == "changed":
+            statusMsg = (
+                self.tr("would reformat")
+                if self.__action
+                in (BlackFormattingAction.Check, BlackFormattingAction.Diff)
+                else self.tr("reformatted")
+            )
+            self.__statistics.changeCount += 1
+
+        elif status == "unchanged":
+            statusMsg = self.tr("unchanged")
+            self.__statistics.sameCount += 1
+
+        elif status == "unmodified":
+            statusMsg = self.tr("unmodified")
+            self.__statistics.sameCount += 1
+
+        elif status == "ignored":
+            statusMsg = self.tr("ignored")
+            self.__statistics.ignoreCount += 1
+
+        elif status == "failed":
+            statusMsg = self.tr("failed")
+            self.__statistics.failureCount += 1
+            isError = True
+
+        else:
+            statusMsg = self.tr("invalid status ({0})").format(status)
+            self.__statistics.failureCount += 1
+            isError = True
+
+        if status != "ignored":
+            self.__statistics.processedCount += 1
+
+        if self.__project:
+            filename = self.__project.getRelativePath(filename)
+
+        itm = QTreeWidgetItem(self.resultsList, [statusMsg, filename])
+        if data:
+            itm.setData(
+                0, BlackFormattingDialog.DataTypeRole, "error" if isError else "diff"
+            )
+            itm.setData(0, BlackFormattingDialog.DataRole, data)
+
+        self.progressBar.setValue(self.progressBar.value() + 1)
+
+        QCoreApplication.processEvents()
+
+
+@dataclass
+class BlackStatistics:
+    """
+    Class containing the reformatting statistic data.
+    """
+
+    ignoreCount: int = 0
+    changeCount: int = 0
+    sameCount: int = 0
+    failureCount: int = 0
+    processedCount: int = 0
+
+
+class BlackReport(QObject, black.Report):
     """
     Class extending the black Report to work with our dialog.
+
+    @signal result(status, file name, data) emitted to signal the reformatting result
+        as three strings giving the status (one of 'changed', 'unchanged', 'unmodified',
+        'failed' or 'ignored'), the file name and data related to the result
     """
 
-    def __init__(self, dialog):
+    result = pyqtSignal(str, str, str)
+
+    def __init__(self, parent=None):
         """
         Constructor
 
-        @param dialog reference to the result dialog
-        @type QDialog
+        @param parent reference to the parent object (defaults to None
+        @type QObject (optional)
         """
-        super().__init__()
-
-        self.ignored_count = 0
-
-        self.__dialog = dialog
+        QObject.__init__(self, parent)
+        black.Report.__init__(self)
 
     def done(self, src, changed, diff=""):
         """
@@ -401,25 +593,15 @@ class BlackReport(black.Report):
         @type str
         """
         if changed is black.Changed.YES:
-            status = (
-                QCoreApplication.translate("BlackFormattingDialog", "would reformat")
-                if self.check or self.diff
-                else QCoreApplication.translate("BlackFormattingDialog", "reformatted")
-            )
-            self.change_count += 1
+            status = "changed"
 
         elif changed is black.Changed.NO:
-            status = QCoreApplication.translate("BlackFormattingDialog", "unchanged")
-            self.same_count += 1
+            status = "unchanged"
 
         elif changed is black.Changed.CACHED:
-            status = QCoreApplication.translate("BlackFormattingDialog", "unmodified")
-            self.same_count += 1
+            status = "unmodified"
 
-        if self.diff:
-            self.__dialog.addResultEntry(status, str(src), data=diff)
-        else:
-            self.__dialog.addResultEntry(status, str(src))
+        self.result.emit(status, str(src), diff)
 
     def failed(self, src, message):
         """
@@ -430,10 +612,7 @@ class BlackReport(black.Report):
         @param message error message
         @type str
         """
-        status = QCoreApplication.translate("BlackFormattingDialog", "failed")
-        self.failure_count += 1
-
-        self.__dialog.addResultEntry(status, str(src), isError=True, data=message)
+        self.result.emit("failed", str(src), message)
 
     def path_ignored(self, src, message=""):
         """
@@ -444,7 +623,84 @@ class BlackReport(black.Report):
         @param message ignore message (default to "")
         @type str (optional)
         """
-        status = QCoreApplication.translate("BlackFormattingDialog", "ignored")
-        self.ignored_count += 1
+        self.result.emit("ignored", str(src), "")
 
-        self.__dialog.addResultEntry(status, str(src))
+
+@dataclass
+class BlackMultiprocessingResult:
+    """
+    Class containing the reformatting result data.
+
+    This class is used when reformatting multiple files in parallel using processes.
+    """
+
+    status: str = ""
+    filename: str = ""
+    data: str = ""
+
+
+class BlackMultiprocessingReport(black.Report):
+    """
+    Class extending the black Report to work with multiprocessing.
+    """
+
+    def __init__(self, resultQueue):
+        """
+        Constructor
+
+        @param resultQueue reference to the queue to put the results into
+        @type multiprocessing.Queue
+        """
+        super().__init__()
+
+        self.__queue = resultQueue
+
+    def done(self, src, changed, diff=""):
+        """
+        Public method to handle the end of a reformat.
+
+        @param src name of the processed file
+        @type pathlib.Path
+        @param changed change status
+        @type black.Changed
+        @param diff unified diff of potential changes (defaults to "")
+        @type str
+        """
+        if changed is black.Changed.YES:
+            status = "changed"
+
+        elif changed is black.Changed.NO:
+            status = "unchanged"
+
+        elif changed is black.Changed.CACHED:
+            status = "unmodified"
+
+        self.__queue.put(
+            BlackMultiprocessingResult(status=status, filename=str(src), data=diff)
+        )
+
+    def failed(self, src, message):
+        """
+        Public method to handle a reformat failure.
+
+        @param src name of the processed file
+        @type pathlib.Path
+        @param message error message
+        @type str
+        """
+        self.__queue.put(
+            BlackMultiprocessingResult(status="failed", filename=str(src), data=message)
+        )
+
+    def path_ignored(self, src, message=""):
+        """
+        Public method handling an ignored path.
+
+        @param src name of the processed file
+        @type pathlib.Path or str
+        @param message ignore message (default to "")
+        @type str (optional)
+        """
+        self.__queue.put(
+            BlackMultiprocessingResult(status="ignored", filename=str(src), data="")
+        )
