@@ -407,12 +407,7 @@ class CondaPackagesWidget(QWidget, Ui_CondaPackagesWidget):
                         for detail in result[package]:
                             version = detail["version"]
                             build = detail["build"]
-                            if "subdir" in detail:
-                                platform = detail["subdir"]
-                            elif "platform" in detail:
-                                platform = detail["platform"]
-                            else:
-                                platform = ""
+                            platform = detail.get("subdir", detail.get("platform", ""))
                             citm = QTreeWidgetItem(itm, ["", version, build, platform])
                             citm.setData(0, self.PackageDetailedDataRole, detail)
 
