@@ -360,6 +360,13 @@ def getFoundDevices():
                         )
                     unknownPorts.append((vid, pid, desc, port.portName()))
 
+        elif bool(port.portName()):
+            # no VID and/or PID available (e.g. in Linux container of ChromeOS)
+            desc = port.description()
+            if not desc:
+                desc = QCoreApplication.translate("MicroPythonDevice", "Unknown Device")
+            unknownPorts.append((0, 0, desc, port.portName()))
+
     return foundDevices, unknownDevices, unknownPorts
 
 
