@@ -636,11 +636,15 @@ class Pip(QObject):
                 proc = QProcess()
                 proc.start(interpreter, args)
                 if proc.waitForStarted(15000) and proc.waitForFinished(30000):
-                    output = str(
-                        proc.readAllStandardOutput(),
-                        Preferences.getSystem("IOEncoding"),
-                        "replace",
-                    ).strip().splitlines()[0]
+                    output = (
+                        str(
+                            proc.readAllStandardOutput(),
+                            Preferences.getSystem("IOEncoding"),
+                            "replace",
+                        )
+                        .strip()
+                        .splitlines()[0]
+                    )
                     try:
                         jsonList = json.loads(output)
                     except Exception:
@@ -703,11 +707,15 @@ class Pip(QObject):
                 proc = QProcess()
                 proc.start(interpreter, args)
                 if proc.waitForStarted(15000) and proc.waitForFinished(30000):
-                    output = str(
-                        proc.readAllStandardOutput(),
-                        Preferences.getSystem("IOEncoding"),
-                        "replace",
-                    ).strip().splitlines()[0]
+                    output = (
+                        str(
+                            proc.readAllStandardOutput(),
+                            Preferences.getSystem("IOEncoding"),
+                            "replace",
+                        )
+                        .strip()
+                        .splitlines()[0]
+                    )
                     try:
                         jsonList = json.loads(output)
                     except Exception:
