@@ -539,6 +539,18 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
                         ],
                     }
                 )
+            if "AllowStarArgAny" not in self.__data["AnnotationsChecker"]:
+                # AllowStarArgAny is the sentinel for the second extension
+                self.__data["AnnotationsChecker"].update(
+                    {
+                        "AllowStarArgAny": AnnotationsCheckerDefaultArgs[
+                            "AllowStarArgAny"
+                        ],
+                        "ForceFutureAnnotations": AnnotationsCheckerDefaultArgs[
+                            "ForceFutureAnnotations"
+                        ],
+                    }
+                )
 
         if "SecurityChecker" not in self.__data:
             from .Security.SecurityDefaults import SecurityDefaults
@@ -628,6 +640,12 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
         self.mypyInitReturnCheckBox.setChecked(
             self.__data["AnnotationsChecker"]["MypyInitReturn"]
         )
+        self.allowStarArgAnyCheckBox.setChecked(
+            self.__data["AnnotationsChecker"]["AllowStarArgAny"]
+        )
+        self.forceFutureAnnotationsCheckBox.setChecked(
+            self.__data["AnnotationsChecker"]["ForceFutureAnnotations"]
+        )
         self.dispatchDecoratorEdit.setText(
             ", ".join(self.__data["AnnotationsChecker"]["DispatchDecorators"])
         )
@@ -805,6 +823,10 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
                 "AllowUntypedDefs": self.allowUntypedDefsCheckBox.isChecked(),
                 "AllowUntypedNested": self.allowUntypedNestedCheckBox.isChecked(),
                 "MypyInitReturn": self.mypyInitReturnCheckBox.isChecked(),
+                "AllowStarArgAny": self.allowStarArgAnyCheckBox.isChecked(),
+                "ForceFutureAnnotations": (
+                    self.forceFutureAnnotationsCheckBox.isChecked()
+                ),
                 "DispatchDecorators": [
                     d.strip() for d in self.dispatchDecoratorEdit.text().split(",")
                 ],
@@ -1246,6 +1268,10 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
                     "AllowUntypedDefs": self.allowUntypedDefsCheckBox.isChecked(),
                     "AllowUntypedNested": self.allowUntypedNestedCheckBox.isChecked(),
                     "MypyInitReturn": self.mypyInitReturnCheckBox.isChecked(),
+                    "AllowStarArgAny": self.allowStarArgAnyCheckBox.isChecked(),
+                    "ForceFutureAnnotations": (
+                        self.forceFutureAnnotationsCheckBox.isChecked()
+                    ),
                     "DispatchDecorators": [
                         d.strip() for d in self.dispatchDecoratorEdit.text().split(",")
                     ],
@@ -1640,6 +1666,22 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
                 )
             )
         )
+        self.allowStarArgAnyCheckBox.setChecked(
+            Preferences.toBool(
+                Preferences.getSettings().value(
+                    "PEP8/AllowStarArgAny",
+                    AnnotationsCheckerDefaultArgs["AllowStarArgAny"],
+                )
+            )
+        )
+        self.forceFutureAnnotationsCheckBox.setChecked(
+            Preferences.toBool(
+                Preferences.getSettings().value(
+                    "PEP8/ForceFutureAnnotations",
+                    AnnotationsCheckerDefaultArgs["ForceFutureAnnotations"],
+                )
+            )
+        )
         self.dispatchDecoratorEdit.setText(
             ", ".join(
                 Preferences.toList(
@@ -1872,6 +1914,13 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
         Preferences.getSettings().setValue(
             "PEP8/MypyInitReturn", self.mypyInitReturnCheckBox.isChecked()
         )
+        Preferences.getSettings().setValue(
+            "PEP8/AllowStarArgAny", self.allowStarArgAnyCheckBox.isChecked()
+        )
+        Preferences.getSettings().setValue(
+            "PEP8/ForceFutureAnnotations",
+            self.forceFutureAnnotationsCheckBox.isChecked(),
+        )
         Preferences.getSettings().setValue(
             "PEP8/DispatchDecorators",
             [d.strip() for d in self.dispatchDecoratorEdit.text().split(",")],
@@ -2013,6 +2062,13 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
         Preferences.getSettings().setValue(
             "PEP8/MypyInitReturn", AnnotationsCheckerDefaultArgs["MypyInitReturn"]
         )
+        Preferences.getSettings().setValue(
+            "PEP8/AllowStarArgAny", AnnotationsCheckerDefaultArgs["AllowStarArgAny"]
+        )
+        Preferences.getSettings().setValue(
+            "PEP8/ForceFutureAnnotations",
+            AnnotationsCheckerDefaultArgs["ForceFutureAnnotations"],
+        )
         Preferences.getSettings().setValue(
             "PEP8/DispatchDecorators",
             AnnotationsCheckerDefaultArgs["DispatchDecorators"],
