@@ -100,15 +100,14 @@ class SubstyledLexer(Lexer):
                                 "Scintilla/{0}/style{1}/substyle{2}/"
                             ).format(self.language(), baseStyle, subStyle)
                             if settings.contains(substyleKey + "Description"):
-                                subStyleData = {}
-                                subStyleData["Description"] = settings.value(
-                                    substyleKey + "Description", ""
-                                )
-                                subStyleData["Words"] = settings.value(
-                                    substyleKey + "Words", ""
-                                )
-                                style = {}
+                                subStyleData = {
+                                    "Description": settings.value(
+                                        substyleKey + "Description", ""
+                                    ),
+                                    "Words": settings.value(substyleKey + "Words", ""),
+                                }
 
+                                style = {}
                                 key = substyleKey + "fore"
                                 if settings.contains(key):
                                     style["fore"] = int(settings.value(key))
