@@ -321,7 +321,9 @@ class BrowserModel(QAbstractItemModel):
             # just ignore the situation we don't have a reference to the item
             return
 
-        dirFilter = QDir.Filter.AllEntries | QDir.Filter.NoDotAndDotDot
+        dirFilter = (
+            QDir.Filter.AllEntries | QDir.Filter.NoDotAndDotDot | QDir.Filter.Hidden
+        )
 
         for itm in self.watchedItems[path]:
             oldCnt = itm.childCount()
@@ -548,7 +550,9 @@ class BrowserModel(QAbstractItemModel):
 
         qdir = QDir(parentItem.dirName())
 
-        dirFilter = QDir.Filter.AllEntries | QDir.Filter.NoDotAndDotDot
+        dirFilter = (
+            QDir.Filter.AllEntries | QDir.Filter.NoDotAndDotDot | QDir.Filter.Hidden
+        )
         entryInfoList = qdir.entryInfoList(dirFilter)
         if len(entryInfoList) > 0:
             if repopulate:
