@@ -277,11 +277,12 @@ class SessionManager(QObject):
         if os.path.exists(self.__lastActiveSession):
 
             if os.path.exists(self.__sessionBackup1):
-                os.unlink(self.__sessionBackup2)
-                shutil.copy(self.__sessionBackup1, self.__sessionBackup2)
+                if os.path.exists(self.__sessionBackup2):
+                    os.remove(self.__sessionBackup2)
+                shutil.copy2(self.__sessionBackup1, self.__sessionBackup2)
+                os.remove(self.__sessionBackup1)
 
-            os.unlink(self.__sessionBackup1)
-            shutil.copy(self.__lastActiveSession, self.__sessionBackup1)
+            shutil.copy2(self.__lastActiveSession, self.__sessionBackup1)
 
     def sessionMetaData(self, includeBackups=False):
         """
@@ -322,7 +323,7 @@ class SessionManager(QObject):
         if self.__sessionMetaData:
             return
 
-        sessionFiles = pathlib.Path(self.getSessionsDirectory()).glob("*.json")
+        sessionFiles = list(pathlib.Path(self.getSessionsDirectory()).glob("*.json"))
 
         for sessionFile in sessionFiles:
             sessionData = self.readSessionFromFile(sessionFile.resolve())
