@@ -1199,13 +1199,13 @@ class WebBrowserTabWidget(EricTabWidget):
         @return dictionary containing the session data
         @rtype dict
         """
-        sessionData = {}
+        sessionData = {
+            # 1. current index
+            "CurrentTabIndex": self.currentIndex(),
+            # 2. tab data
+            "Tabs": [],
+        }
 
-        # 1. current index
-        sessionData["CurrentTabIndex"] = self.currentIndex()
-
-        # 2. tab data
-        sessionData["Tabs"] = []
         for index in range(self.count()):
             browser = self.widget(index)
             data = browser.getSessionData()
